<?php

/**
 * This file contains PCSG\API\Ebay
 */

namespace PCSG\Api;

/**
 * PCSG eBay API
 * Mini API access
 *
 * @author www.pcsg.de (Henning Leutz)
 */

class Ebay extends \QUI\QDOM
{
    /**
     * Internal Session-ID
     *
     * @var String
     */
    protected $_Session;

    /**
     * Constructor
     * @param array $attributes - optional
     */
    public function __construct($attributes)
    {
        // defaults
        $this->setAttributes(array(
            'SECURITY-APPNAME'     => '',
            'GLOBAL-ID'            => 'EBAY-DE',
            'OPERATION-NAME'       => 'findItemsAdvanced',
            'SERVICE-VERSION'      => '1.11.0',
            'REST-PAYLOAD'         => '',

            'affiliate.networkId'  => 9,
            'affiliate.trackingId' => '5336210971',
            'affiliate.customId'   => '5336210971',

            'sandbox' => false,
            'debug'   => false,

            // for authentication
            'DEV-NAME'            => '',
            'CERT-NAME'           => '',
            'COMPATIBILITY-LEVEL' => 681,
            'CALL-NAME'           => 'FetchToken',
            'RuName'              => ''
        ));

        // set options
        $this->setAttributes( $attributes );

        // check app id
        if ( !$this->getAttribute('SECURITY-APPNAME') )
        {
            throw new \Exception(
                'No Access. No eBay SECURITY-APPNAME given',
                401
            );
        }
    }

    /**
     * Make a normal eBay request with the set attributes
     *
     * @return Array
     */
    public function search()
    {
        $url   = 'http://svcs.ebay.com/services/search/FindingService/v1?';
        $attr  = $this->_getFindingServiceAttributes();
        $parts = array();

        foreach ( $attr as $key => $value ) {
            $parts[] = sprintf( '%s=%s', $key, rawurlencode($value) );
        }

        $get  = file_get_contents( $url . implode( '&', $parts ) );
        $data = json_decode( $get, true );

        return $this->_parseItems( $data );
    }

    /**
     * Return the Articles from the seller
     *
     * @param String $sellerid - seller id, seller name
     * @return Array
     */
    public function getItemsBySeller($sellerid=false)
    {
        $url  = 'http://svcs.ebay.com/services/search/FindingService/v1?';
        $attr = $this->_getFindingServiceAttributes();

        $attr['itemFilter(0).name']     = 'Seller';
        $attr['itemFilter(0).value(0)'] = $sellerid;
        $attr['itemFilter(1).name'] = 'HideDuplicateItems';
        $attr['itemFilter(1).value(1)'] = '1';

        $parts = array();

        foreach ( $attr as $key => $value ) {
            $parts[] = sprintf( '%s=%s', $key, rawurlencode($value) );
        }

        $get = file_get_contents( $url . implode( '&', $parts ) );

//         $get  = file_get_contents( $url . http_build_query( $attr ) );
        $data = json_decode( $get, true );

        return $this->_parseItems( $data );
    }

    /**
     * Return the
     *
     * @return Array
     */
    protected function _getFindingServiceAttributes()
    {
        $attr = $this->getAttributes();

        // old api compatiblity
        $attr['RESPONSE-DATA-FORMAT'] = 'JSON';

        if ( !isset( $attr['paginationInput.entriesPerPage'] ) ) {
            $attr['paginationInput.entriesPerPage'] = 10;
        }

        if ( !isset( $attr['outputSelector'] ) ) {
            $attr['outputSelector'] = 'PictureURLLarge';
        }

        if ( !isset( $attr['keywords'] ) ) {
            $attr['keywords'] = '';
        }

        return $attr;
    }

    /**
     * Parse the return of a FindingService request
     *
     * @return Array
     */
    protected function _parseItems( $data )
    {
        if ( isset( $data['findItemsAdvancedResponse'] ) )
        {
            $_data = $data['findItemsAdvancedResponse'];

        } else if ( isset( $data['findItemsByKeywordsResponse'] ) )
        {
            $_data = $data['findItemsByKeywordsResponse'];
        }


        if ( !isset( $_data[0] ) ) {
            return array();
        }

        if ( isset( $_data[0]['ack'] ) &&
             $_data[0]['ack'][0] == 'Failure' )
        {

            throw new \Exception(
                $_data[0]['errorMessage'][0]['error'][0]['message'][0]
            );
        }

        if ( !isset( $_data[0]['searchResult'] ) ||
             !isset( $_data[0]['searchResult'][0] ) )
        {
            return array();
        }

        if ( $_data[0]['searchResult'][0]['@count'] == 0 ) {
            return array();
        }

        $result = array();
        $list   = $_data[0]['searchResult'][0]['item'];

        foreach ( $list as $item )
        {
            if ( !isset( $item['galleryURL'] ) ) {
                $item['galleryURL'][0] = '';
            }

            if ( !isset( $item['pictureURLLarge'] ) ) {
                $item['pictureURLLarge'][0] = $item['galleryURL'][0];
            }

            if ( !isset( $item['shippingInfo'] ) ||
                 !isset( $item['shippingInfo'][0]['shippingServiceCost'] ) ||
                 !isset( $item['shippingInfo'][0]['shippingServiceCost'][0]['__value__'] ) )
            {
                $item['shippingInfo'][0]['shippingServiceCost'][0]['__value__'] = '';
            }

            if ( !isset( $item['shippingInfo'] ) ||
                 !isset( $item['shippingInfo'][0]['shippingServiceCost'] ) ||
                 !isset( $item['shippingInfo'][0]['shippingServiceCost'][0]['@currencyId'] ) )
            {
                $item['shippingInfo'][0]['shippingServiceCost'][0]['@currencyId'] = '';
            }

            if ( !isset( $item['sellingStatus'] ) ||
                 !isset( $item['sellingStatus'][0]['convertedCurrentPrice'] ) ||
                 !isset( $item['sellingStatus'][0]['convertedCurrentPrice'][0]['__value__'] ) )
            {
                $item['sellingStatus'][0]['convertedCurrentPrice'][0]['__value__'] = '';
            }

            if ( !isset( $item['sellingStatus'] ) ||
                 !isset( $item['sellingStatus'][0]['convertedCurrentPrice'] ) ||
                 !isset( $item['sellingStatus'][0]['convertedCurrentPrice'][0]['@currencyId'] ) )
            {
                $item['sellingStatus'][0]['convertedCurrentPrice'][0]['@currencyId'] = '';
            }



            $result[] = array(
                'itemId'      => $item['itemId'][0],
                'title'       => $item['title'][0],
                'endTime'     => $item['listingInfo'][0]['endTime'][0],
                'url'         => $item['viewItemURL'][0],
                'listingType' => $item['listingInfo'][0]['listingType'],
                'imageUrl'    => $item['pictureURLLarge'][0],
                'galleryURL'  => $item['galleryURL'][0],

                'primaryCategoryId'   => $item['primaryCategory'][0]['categoryId'][0],
                'primaryCategoryName' => $item['primaryCategory'][0]['categoryName'][0],

                'price' => array(
                    'value'    => \QUI\Utils\Convert::formPrice(
                        $item['sellingStatus'][0]['convertedCurrentPrice'][0]['__value__'], 3
                    ),
                    'currency' => $item['sellingStatus'][0]['convertedCurrentPrice'][0]['@currencyId'],
                ),

                'timeLeft' => $this->getPrettyTimeFromEbayTime(
                    $item['sellingStatus'][0]['timeLeft'][0]
                ),

                'shippingCost' => array(
                    'value'    => \QUI\Utils\Convert::formPrice(
                        $item['shippingInfo'][0]['shippingServiceCost'][0]['__value__'], 3
                    ),
                    'currency' => $item['shippingInfo'][0]['shippingServiceCost'][0]['@currencyId']
                )
            );
        }

        return $result;
    }

    /**
     * Convert the eBay timestamp to a pretty format
     *
     * @param $eBayTimeString Input is of form 'PT12M25S'
     * @return string
     */
    static function getPrettyTimeFromEbayTime($eBayTimeString)
    {
        $matchAry = array();
        $pattern  = "#P([0-9]{0,3}D)?T([0-9]?[0-9]H)?([0-9]?[0-9]M)?([0-9]?[0-9]S)#msiU";
        preg_match( $pattern, $eBayTimeString, $matchAry );

        $days  = false;
        $hours = false;
        $min   = false;
        $sec   = false;

        if ( isset( $matchAry[1] ) ) {
            $days = (int)$matchAry[1];
        }

        if ( isset( $matchAry[2] ) ) {
            $hours = (int)$matchAry[2];
        }

        if ( isset( $matchAry[3] ) ) {
            $min = (int)$matchAry[3];
        }

        if ( isset( $matchAry[4] ) ) {
            $sec = (int)$matchAry[4];
        }


        $retnStr = '';

        if ( $days ) {
            $retnStr .= "$days Tag(e)";
        }

        if ( $hours ) {
            $retnStr .= " $hours Stunde(n)";
        }

        if ( $min ) {
            $retnStr .= " $min Minute(n)";
        }

        if ( $sec ) {
            $retnStr .= " $sec Sekunde(n)";
        }

        return $retnStr;
    }

    /**
     * If a session already exist you can set the internal Session-ID
     *
     * @param String $SessID
     */
    public function setSession($SessID)
    {
        $this->_Session = $SessID;
    }

    /**
     * Create a ebay session, if a session is set, this session would be return
     *
     * @return String
     */
    public function getSession()
    {
        if ( !empty( $this->_Session ) ) {
            return $this->_Session;
        }

        // get session id
        $this->setAttribute( 'CALL-NAME', 'GetSessionID' );

        $Response = $this->_sendHttpRequest(
            '<?xml version="1.0" encoding="utf-8" ?>'.
            '<GetSessionIDRequest xmlns="urn:ebay:apis:eBLBaseComponents">'.
                '<Version>'. $this->getAttribute( 'COMPATIBILITY-LEVEL' ) .'</Version>'.
                '<RuName>'. $this->getAttribute( 'RuName' ) .'</RuName>'.
            '</GetSessionIDRequest>'
        );

        $this->_debug( $Response );


        $ResponseXML = simplexml_load_string( $Response );

        $this->setSession( (string)$ResponseXML->SessionID );

        return $this->_Session;
    }

    /**
     * Ask for a token
     */
    public function getToken()
    {
        $this->setAttribute( 'CALL-NAME', 'FetchToken' );

        $response = $this->_sendHttpRequest(
            '<?xml version="1.0" encoding="utf-8" ?>'.
            '<FetchTokenRequest xmlns="urn:ebay:apis:eBLBaseComponents">'.
                '<SessionID>'. $this->getSession() .'</SessionID>'.
            '</FetchTokenRequest>'
        );

        $this->_debug( $response );


        $ResponseXML = simplexml_load_string( $response );

        // if login not allowed
        // then ask for it
        if ( $ResponseXML->Ack == 'Failure' ) {
            throw new \Exception( 'Get Token not allowed' );
        }

        $Token      = (string)$ResponseXML->eBayAuthToken;
        $expiration = $ResponseXML->HardExpirationTime;

        return $Token;
    }

    /**
     * Fetch the signin form for the session
     *
     * @return String
     */
    public function signInForm()
    {
        return file_get_contents( $this->signInUrl() );
    }

    /**
     * Return the signin Url
     *
     * @return string
     */
    public function signInUrl()
    {
        $url = 'https://signin.ebay.com/ws/eBayISAPI.dll?';

        if ( $this->getAttribute( 'sandbox' ) ) {
            $url = 'https://signin.sandbox.ebay.com/ws/eBayISAPI.dll?';
        }

        $attr = array(
            'RuName' => $this->getAttribute('RuName'),
            'SessID' => $this->getSession()
        );

        return $url . http_build_query( $attr );
    }

    /**
     * Return the difference between two dates
     *
     * @param {String} $start - start date, ebay date
     * @param {String} $end - end date, ebay date
     */
    public function dateDiff( $start, $end=false )
    {
        $return = array();

        try
        {
            $start = new \DateTime( $start );
            $end   = new \DateTime( $end );

            $form  = $start->diff( $end );

        } catch ( \Exception $Exception )
        {
            return $Exception->getMessage();
        }

        $hour = sprintf( "%02d", $form->h );
        $min  = sprintf( "%02d", $form->i );
        $sec  = sprintf( "%02d", $form->s );

        $year  = sprintf( "%02d", $form->y );
        $month = sprintf( "%02d", $form->m );
        $days  = sprintf( "%02d", $form->d );

        $result = '';

        if ( $form->y || $form->m || $form->d )
        {
            if ( !$form->y || !$form->m || $form->d )
            {
                $result = "{$days} Tage, ";
            } else
            {
                $result = "{$days} Tage, {$month} Monate, {$year} Jahre ";
            }
        }

        $result .= "{$hour}:{$min}:{$sec}";


        return $result;
    }


    /**
     * Build the ebay request header for authentication
     *
     * @return Array
     */
    protected function _buildHeaders()
    {
        return array(
            // Regulates versioning of the XML interface for the API
            'X-EBAY-API-COMPATIBILITY-LEVEL: '. $this->getAttribute( 'COMPATIBILITY-LEVEL' ),

            //set the keys
            'X-EBAY-API-DEV-NAME: '. $this->getAttribute('DEV-NAME'),
            'X-EBAY-API-APP-NAME: '. $this->getAttribute('SECURITY-APPNAME'),
            'X-EBAY-API-CERT-NAME: '. $this->getAttribute('CERT-NAME'),

            'X-EBAY-API-CALL-NAME: '. $this->getAttribute( 'CALL-NAME' ),

            'X-EBAY-API-SITEID: '. $this->_globalIDToSiteID(
                $this->getAttribute( 'GLOBAL-ID' )
            )
        );
    }

    /**
     * Return the eBay Site-ID of the eBay Global-ID
     *
     * @param String $globalID,
     * @return Integer
     */
    protected function _globalIDToSiteID($globalID)
    {
        $ids = array(
            'EBAY-AT'    => 16,
            'EBAY-AU'    => 15,
            'EBAY-CH'    => 193,
            'EBAY-DE'    => 77,
            'EBAY-ENCA'  => 2,
            'EBAY-ES'    => 186,
            'EBAY-FR'    => 71,
            'EBAY-FRBE'  => 23,
            'EBAY-FRCA'  => 210,
            'EBAY-GB'    => 3,
            'EBAY-HK'    => 201,
            'EBAY-IE'    => 205,
            'EBAY-IN'    => 203,
            'EBAY-IT'    => 101,
            'EBAY-MOTOR' => 100,
            'EBAY-MY'    => 207,
            'EBAY-NL'    => 146,
            'EBAY-NLBE'  => 123,
            'EBAY-PH'    => 211,
            'EBAY-PL'    => 212,
            'EBAY-SG'    => 216,
            'EBAY-US'    => 0
        );

        return isset( $ids[ $globalID ] ) ? $ids[ $globalID ] : 77;
    }

    /**
     * Send an http request to eBay
     *
     * @param String $requestBody - XML request
     * @return mixed
     */
    protected function _sendHttpRequest($requestBody)
    {
        $serverUrl   = 'https://api.ebay.com/ws/api.dll';
        $shoppingURL = 'http://open.api.ebay.com/shopping';
        $findingURL  = 'http://svcs.ebay.com/services/search/FindingService/v1';

        if ( $this->getAttribute( 'sandbox' ) )
        {
            $serverUrl   = 'https://api.sandbox.ebay.com/ws/api.dll';
            $shoppingURL = 'http://open.api.sandbox.ebay.com/shopping';
            $findingURL  = 'http://svcs.sandbox.ebay.com/services/search/FindingService/v1';
        }

        $Curl = curl_init();

        curl_setopt( $Curl, CURLOPT_SSL_VERIFYPEER, 0 );
        curl_setopt( $Curl, CURLOPT_SSL_VERIFYHOST, 0 );

        curl_setopt( $Curl, CURLOPT_RETURNTRANSFER, 1 );
        curl_setopt( $Curl, CURLOPT_POST, 1 );

        curl_setopt( $Curl, CURLOPT_URL, $serverUrl );
        curl_setopt( $Curl, CURLOPT_HTTPHEADER, $this->_buildHeaders() );
        curl_setopt( $Curl, CURLOPT_POSTFIELDS, $requestBody );

        $Response = curl_exec( $Curl );

        curl_close( $Curl );

        return $Response;
    }

    /**
     * Log a message via error_log
     * only if debug is true
     *
     * @param String $message - the messsage that would be log
     */
    public function _debug($message)
    {
        if ( $this->getAttribute( 'debug' ) ) {
            error_log( "\n\n".$message );
        }
    }
}
